import os
import six
import docker_emperor.logger as logger


__all__ = ['run']


def run(root, *args, **kwargs):

    name = args[0].strip() if args else None
    if name:
        if name in root.project['mounting']:
            root.project.config['mounting'] = name
            logger.success(u'Mounting <b>%s</b> selected.' % root.mounting.name)
        else:
            logger.error(u'Mounting <b>%s</b> unknow.' % name)
            exit(1)

    else:
        def select_mounting_name():
            # logger.ask('0) no mounting (localhost)')
            logger.ask(u'Please select the <b>{}</b> mounting to work on'.format(root.project.name))
            for i, m in enumerate(root.project['mounting']):
                logger.choice(u'<b>{}</b>] {}'.format(i+1, m.name))
            mi = six.moves.input(': ')
            try:
                if mi == '0':
                    raise Exception
                return root.project['mounting'][int(mi)-1].name
            except Exception as e:
                logger.error(u'<b>%s</b> is not a valid choice' % mi)
                return select_mounting_name()

        root.project.config['mounting'] = select_mounting_name()
        logger.success(u'Mounting <b>%s</b> selected.' % root.mounting.name)



import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    
    cmd = root.bash(
        root.machine.bin, 
        'status', 
        root.machine.name, 
        *args,
        sys=True
    )
    logger.info('Machine <b>%s</b> status: <b>%s</b>.' % (root.machine.name, cmd.out))
    return cmd.out


import six
import os
import subprocess
import docker_emperor.logger as logger
import tempfile


__all__ = ['Command']


COMPOSE_COMMANDS = [
    'build',
    'bundle',
    'config',
    'create',
    'down',
    'events',
    'exec',
    'help',
    'kill',
    'logs',
    'pause',
    'port',
    'ps',
    'pull',
    'push',
    'restart',
    'rm',
    'run',
    'scale',
    'start',
    'stop',
    'top',
    'unpause',
    'up',
]

MACHINE_COMMANDS = [
    'active',
    'config',
    'create',
    'env',
    'help',
    'inspect',
    'ip',
    'kill',
    'ls',
    'mount',
    'provision',
    'regenerate-certs',
    'restart',
    'rm',
    'scp',
    'ssh',
    'start',
    'status',
    'stop',
    'upgrade',
    'url',
]



class Command():

    verbose = 0

    def __repr__(self):
        return self.cmd_line

    def __init__(self, *args, **kwargs):

        self.args = list(args)
        self.env = kwargs.pop('env', [])
        self.is_system = kwargs.pop('is_system', False)
        self.compose = kwargs.pop('compose', None)
        self.silently_fail = kwargs.pop('silently_fail', False)
        self.flat_args = kwargs.pop('flat_args', False)
        self.escape = kwargs.pop('escape', True)
        self.error = None

        for i, arg in enumerate(self.args):
            if isinstance(arg, six.integer_types):
                arg = str(arg)
            if not isinstance(arg, six.string_types):
                raise Exception("Argument {} is invalid: {} is not string types".format(i, arg))

            key_val = arg.split('=')
            if len(key_val) == 2:
                key, val = key_val
                val = val.strip('"').replace('\\"', '"').replace('"', '\\"')
                if ' ' in val:
                    val = '"%s"' % val.replace('\\ ', ' ').replace(' ', '\\ ')
                arg = '%s=%s' % (key, val)
            self.args[i] = arg

    @property
    def is_success(self):
        return not bool(self.error)

    def system_cmd(self, cmd):
        ftmp = tempfile.NamedTemporaryFile(suffix='.out', prefix='tmp', delete=False)
        fpath = ftmp.name
        if os.name=="nt":
            fpath = fpath.replace("/","\\") # forwin
        ftmp.close()
        os.system(cmd + " > " + fpath)
        data = ""
        with open(fpath, 'r') as file:
            data = file.read()
            file.close()
        os.remove(fpath)
        return data
    

    def run(self, log=False):

        if not self.env: self.env = []
        if not isinstance(self.env, list):
            raise Exception("Env arguments are invalid: {} is not list types".format(self.env))

        if isinstance(self.compose, Command.Compose):
            self.env += self.compose.mounting.docker_env       

        self.cmd_line = " ".join(self.env + self.args)

        if '--verbose' in self.args:
            self.cmd_line = self.cmd_line.replace('--verbose', '')
        #  logger.comment(self.cmd_line)

        # if kwargs.get('log', True):
        #     logger.comment("Env.\n" + "\n".join(self.env))

        if isinstance(self.compose, Command.Compose):
            for name, value in self.compose.environment:
                self.cmd_line = self.cmd_line.replace('${%s}' % (name), value)

        if self.is_system:
            self.out = os.system(self.cmd_line)
            # if self.out.startswith('ERROR:'):
            #     self.error = self.out

        else:
            self.lines = []
            self.process = subprocess.Popen(
                self.cmd_line, 
                shell=True, 
                stdin=subprocess.PIPE, 
                stdout=subprocess.PIPE, 
                stderr=subprocess.PIPE
            )
            self.out = ''

            for line in iter(self.process.stdout.readline, ''):  # replace '' with b'' for Python 3
                self.out += line
                self.lines.append(line.strip())
                self.last_line = line

            self.out = self.out.strip()
            stdout, stderr = self.process.communicate()
            self.error_code = self.process.returncode
            self.error = stderr.decode('utf-8').strip()

        if self.error:                
            # self.error = self.error.replace('docker-machine ls', 'de ls')
            if not self.silently_fail:
                logger.error(self.error)
                exit(0) 

        return self

    def log(self):
        logger.success(self.out)




import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    root.run_command('machine:start', internal=True)
    cmd = root.bash(
        root.compose.bin,
        'stop',
        *args,
        compose=root.compose,
        is_system=True
    )
    if cmd.is_success:
        logger.success('<b>%s</b> is stopped.' % (root.compose.name, ))


import os
from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    root.run_command('machine:start', internal=True)

    if '--set-hosts' in args:
        root.run_command('hosts:set', internal=True)
        args = filter(lambda s:s != '--set-hosts', args)
    else:
        logger.cmd('Hosts mapping ignored.')

    logger.cmd('Up project <b>%s</b>' % (root.compose.name, ))
    cmd = root.bash(
        root.compose.bin,
        'up',
        *args,
        compose=root.compose,
        is_system=True
    )
    if cmd.is_success:
        logger.success('<b>%s</b> is up.' % (root.compose.name, ))

    if root.mounting['hosts']:
        for host in root.mounting['hosts']:
            logger.success('Project is accessible by http://%s.' % (host, ))




import six
import docker_emperor.logger as logger
from docker_emperor.nodes.context import Context


def run(root, *args, **kwargs):

    name = args[0].strip() if args else None
    if name:
        if name in root.project['contexts']:
            root.project.config['context'] = name
            logger.success(u'Context <b>%s</b> selected.' % root.context.name)
        else:
            logger.error(u'Context <b>%s</b> unknow.' % name)
            exit(0)

    else:
        contexts = root.project['contexts']
        if not contexts:
            contexts['default'] = Context('default')
            root.project.config['context'] = 'default'
            logger.warning(u'No context defines, use <b>%s</b>.' % root.context.name)
        else:

            def select_context_name(contexts):
                logger.ask(u'Please select the <b>{}</b> context to work on'.format(root.project.name))
                for i, c in enumerate(contexts):
                    logger.choice(u'<b>%s</b>] %s' % (i+1, c.name))
                ci = six.moves.input(': ')
                try:
                    if ci == '0':
                        raise Exception
                    return contexts[int(ci)-1].name
                except Exception as e:
                    logger.error(u'<b>%s/b> is not a valid choice' % ci)
                    return select_context_name(contexts)

            root.project.config['context'] = select_context_name(contexts)
            logger.success(u'Context <b>%s</b> selected.' % root.context.name)


import os
from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    logger.cmd('Run project <b>%s</b>' % (root.compose.name, ))
    root.run_command('machine:start', internal=True)
    root.run_command('down', internal=True)
    root.run_command('up', *args, internal=True)


import os
import copy
import six
from docker_emperor.nodes.environment import Environment
from docker_emperor.utils import setdefaultdict, yaml


__all__ = ['Services', 'Service']


class Services(dict):

    def __new__(cls, data, *args, **kwargs):
        return dict.__new__(cls, data, *args, **kwargs)

    def __init__(self, data, *args):
        super(Services, self).__init__(setdefaultdict(data, ordered=True))
        for key ,val in self.items():
            self[key] = Service(key, val)
        for arg in args:
            self < arg


    def __gt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        return inst < self

    def __lt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        for key, val in inst.items():
            if not key in self:
                self[key] = val.copy()
            else:
                self[key] < val
        return self

    def __iter__(self):
        for name, service in self.items():
            yield service

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, super(dict, self).__repr__())

    # # For YAML dumper
    # def items(self):
    #     return [ (name, self[name]) for name, value in super(Services, self).items() ]


yaml.add_representer(Services, lambda dumper, data: dumper.represent_dict(data))


class Service(dict):
               

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __init__(self, name, data, *args):
        self.name = name
        super(self.__class__, self).__init__(setdefaultdict(data))
        for default_name, default_class in [
            ('environment', Environment),
        ]:
            self[default_name] = default_class(self[default_name])
        for arg in args:
            self < arg

    def __getitem__(self, key): 
        return self.get(key)

    def __gt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        return inst < self

    def __lt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        for key, val in inst.items():
            if  key == 'environment':
                self[key] < val
            else:
                if not key in self:
                    self[key] = copy.deepcopy(val)
                else:
                    if isinstance(self[key], dict) and isinstance(val, dict):
                        self[key].update(val)
                    elif isinstance(self[key], list) and isinstance(val, list):
                        self[key] = list(set(self[key] + copy.deepcopy(val)))
                    elif isinstance(val, six.string_types):
                        self[key] = val

        return self

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, self.name)
       
    def copy(self):
        return self.__class__(self.name, dict(self))

    

yaml.add_representer(Service, lambda dumper, data: dumper.represent_dict(data))
#print(Service('test1', { 'key1-test1': 1, 'key2-test1': 1}) < Service('test2', { 'key1-test1': 2, 'key3-test2': 1}))
# print(yaml.dump(Service('test1', { 'key1-test1': 1, 'key2-test1': 1})))


import os
import six
import docker_emperor.logger as logger


__all__ = ['run']


def run(root, *args, **kwargs):

    project = None

    project_name = args[0] if len(args) else None
    if project_name:
        project = root.projects.get(project_name)
        if not project:
            logger.warning('Project <b>%s</b> unknow.' % project_name)
            
    if not project:

        def select_project():
            # logger.ask('0) no machine (localhost)')
            logger.ask('Select the project to work on')
            for i, p in enumerate(root.projects.items()):
                key, value = p
                logger.choice('<b>{}]</b> {}'.format(i+1, key))
            pi = six.moves.input(': ')
            try:
                if pi == '0':
                    raise Exception
                return root.projects.items()[int(pi)-1]
            except Exception as e:
                logger.error('{} is not a valid choice'.format(pi))
                return select_project()
        project_name, project = select_project()

    if project:
        workdir = project.get('workdir')
        if workdir and os.path.isdir(workdir):
            logger.success('Workon the project <b>%s</b> in <b>%s</b>' % (project_name, workdir))
            os.chdir(workdir)
            shell = os.environ.get('SHELL', '/bin/sh')
            # root.run_command('info', internal=True)
            os.execl(shell, shell)
            return
            # execl() does not return; it replaces the Python process with a new shell process
    


import os
import sys
import yaml
import tempfile
import json
import collections
import six
import argparse
import subprocess
import imp
from pprint import pprint
from docker_emperor.commands import Command
from docker_emperor.nodes.project import Project
from docker_emperor.exceptions import DockerEmperorException
from docker_emperor.utils import setdefaultdict, combine, yamp_load
import docker_emperor.logger as logger


class Root():

    version = __import__('docker_emperor').__version__

    def __init__(self):
        self.module_root = os.path.dirname(os.path.abspath(__file__))
        self.bin_root = os.path.join(self.module_root, 'bin')        
        self.config_path = os.path.expanduser(os.path.join('~', ".docker-emperor"))
        try:
            self.config = json.load(open(self.config_path, 'rb'))
        except:
            self.config = {}
        self.docker_path = 'docker'
        self.logger = logger.Logger(self)
        self.current_mounting = None

    @property
    def root_path(self):
        return os.getcwd()
    
    
    @property  
    def projects(self):
        n = '__projects'
        if not hasattr(self, n):
            self.config['projects'] = setdefaultdict(self.config.setdefault('projects'))
            setattr(self, n, self.config['projects'])
        return getattr(self, n)

    @property   
    def project(self):
        n = '__project'
        if not hasattr(self, n):
            setattr(self, n, Project(self))
        return getattr(self, n)

    @property
    def mounting(self):
        return self.project.mounting

    @property
    def compose(self):
        return self.project.compose

    def entrypoint(self, cmd=None, *args):
        try:
            args = list(args)
            if cmd:
                self.run_command(cmd, *args)
                self.save_config()
            else:
                logger.info('docker-emperor version {}'.format(self.version))

        except DockerEmperorException as e:
            logger.error(e)

    def run_command(self, name, *args, **kwargs):

        internal = kwargs.pop('internal', False)

        if args and args[-1][0] == '@':
            self.current_mounting = args[-1][1:]
            args = args[0:-1]

        if '--verbose' in args:
            Command.verbose = 1

        try:
            module_name = name.replace(':', '.')
            mod = __import__('docker_emperor.commands.{}'.format(module_name), globals(), locals(), ['run'], 0)
            return mod.run(self, *args, **kwargs)

        except ImportError as e:
            if internal:
                raise ImportError(e)
            else:
                if not self.project.run_command(name, *args, **kwargs):
                    logger.error('Unknown command %s' % name)
            return False

    def bash(self, *args, **kwargs):        
        cmd = Command(*args, **kwargs)
        cmd.run()
        return cmd

    def save_config(self):
        if self.config:
            file = open(self.config_path, 'wb')
            file.write(json.dumps(self.config, indent=4) )
            file.close()

    


# import readline

# def completer(text, state):
#     options = [i for i in commands if i.startswith(text)]
#     if state < len(options):
#         return options[state]
#     else:
#         return None

# readline.parse_and_bind("tab: complete")
# readline.set_completer(completer)

def entrypoint():
    try:
        argsparser = argparse.ArgumentParser(description='Docker emperor orchestrator') 
        argsparser.add_argument('args', nargs=argparse.REMAINDER, action="store") 
        Root().entrypoint(*argsparser.parse_args().args)
    except KeyboardInterrupt:
        logger.warning('Keyboard interrupt, exiting.')
        exit(0)

if __name__ == "__main__": entrypoint()








# # @property
# # def machine(self):
# #     attr = '_machine'
# #     if not hasattr(self, attr): setattr(self, attr, self.machines.select(self.machine_name))
# #     return getattr(self, attr)


# def _run(self, *args):
#     return Command(self, self.compose_path, *args, env=self.machine.docker_env)


# def cmd_start(self, *args):
#     self.cmd_prune()
#     self.compose("down --remove-orphans")
#     self.compose("up", *args)

# def cmd_startd(self, *args):
#     self.cmd_start('-d', *args)

# def cmd_logs(self, *args):
#     self.compose('logs', '-f', *args)


# def cmd_run(self, *args):
#     self.compose('run', *args)

# def cmd_exec(self, *args):
#     self.compose('exec', *args)

# def cmd_sethost(self, host):
#     bin_path = os.path.join(self.module_root, 'bin')
#     self.cmd("docker run -t -i -v {bin_path}/sethost:/bin/sethost -v /etc/hosts:/etc/hosthosts -v /etc/hosts.bak:/etc/hosthosts.bak busybox sh /bin/sethost 0.0.0.0 {}".format(host))

# def cmd_prune(self):
#     self.cmd("docker network prune -f")
#     self.cmd("docker system prune -f")
#     self.cmd("docker volume prune -f")


# def cmd(self, *args):
#     cmd = " ".join(args).strip()
#     self.log_info('> {}'.format(cmd))
#     try:
#         output = subprocess.check_output(cmd.split())
#         print(output.strip())
#     except Exception as e:
#         raise DockerEmperorException()#"error: " + str(e))#, sys.exc_info())


# def log_info(self, message):
#     print('{}{}{}{}'.format(C.YELLOW, C.LYELLOW, message, C.ENDC))

# def log_success(self, message):
#     print('{}{}{}{}'.format(C.GREEN, C.LGREEN, message, C.ENDC))

# def log_error(self, message):
#     print('{}{}{}{}'.format(C.ERROR, C.LERROR, message, C.ENDC))





import os
import six
from docker_emperor.nodes.environment import Environment
from docker_emperor.nodes.mounting import Mountings
from docker_emperor.nodes.command import Commands
from docker_emperor.nodes.compose import Compose 
from docker_emperor.commands import Command 
from docker_emperor.exceptions import DockerEmperorException
from docker_emperor.utils import setdefaultdict,  yamp_load
import docker_emperor.logger as logger


__all__ = ['Project']


class Project(dict):

    FILES = ['docker-emperor.yml', 'docker-compose.yml']

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, self.name)

    def __getitem__(self, key): 
        return self.get(key)

    def __init__(self, root):
        self.root = root
        super(Project, self).__init__(self.get_yml_data())
        for default_name, default_class in [
            ('environment', Environment),
            ('commands', Commands),
        ]:
            self[default_name] = default_class(self[default_name])  

        self['mounting'] = Mountings(self, self['mounting'])

        self.config = setdefaultdict(root.projects, self.name, {})
        self.config['workdir'] = os.path.abspath(self.root.root_path)

    def get_yml_data(self):
        for file in self.FILES:
            filename = os.path.join(self.root.root_path, file)
            if os.path.isfile(filename):
                data = yamp_load(open(filename, 'rb').read())
                if not isinstance(data, dict):
                    raise DockerEmperorException('{} is not yml as dict'.format(os.path.basename(file)))
                return data
        raise DockerEmperorException('{} not found in {}'.format(" or ".join(self.FILES), self.root.root_path))

    @property
    def name(self): 
        n = '__name'
        if not hasattr(self, n):
            setattr(self, n, self.pop('name',  
                self.pop('project_name',                       # default 0
                    self['environment'].get('COMPOSE_PROJECT_NAME',    # default 1
                        os.environ.get('COMPOSE_PROJECT_NAME',      # default 2
                            os.path.basename(self.root.root_path)   # default 3
                        )
                    )
                )
            ))
        return getattr(self, n)

    ''' 
    Run custom project defined commands
    '''
    def run_command(self, name, *args):
        commands = self['commands']
        commands < self.mounting['commands'] 
        if name in commands:                    
            command = commands[name]
            logger.cmd('Run custom command <b>%s</b>' % (command.name, ))
            for line in command.commands:
                if line == name:
                    logger.error('Comand loop error: <b>%s</b>' % (line, ))
                else:
                    # TODO: Improve with @mounting detection and Split by | and < or >
                    if self.root.current_mounting and line.split()[-1][0] != '@':
                        local_args = tuple(args) + ('@%s' % self.root.current_mounting,)
                    else:
                        local_args = tuple(args)

                    for name, value in self.compose.environment:
                        line = line.replace('${%s}' % (name), value)

                    logger.cmd('Run %s %s' % (line, " ".join(local_args)))
                    cmd_args = tuple(line.split()) + local_args
                    cmd = self.root.bash(
                        'docker-emperor',
                        *cmd_args,
                        compose=self.compose,
                        is_system=True
                    )
                    if not cmd.is_success:
                        break
            return True
        return False


    @property
    def compose(self): 
        n = '__compose'
        if not hasattr(self, n):
            setattr(self, n, Compose(self.root))
        return getattr(self, n)

    @property
    def mounting(self):
        n = '__mounting'
        if not hasattr(self, n):
            if not self.root.current_mounting:
                self.root.current_mounting = 'localhost'
            mounting = self['mounting'][self.root.current_mounting]
            if not mounting:
                self.root.run_command('mount', internal=True)
                return self.mounting
            else:
                setattr(self, n, mounting)
        return getattr(self, n)


import os
import six
import docker_emperor.logger as logger


__all__ = ['run']


def run(root, *args, **kwargs):

    machines = root.project['machines']
    for i, m in enumerate(root.project['machines']):
        logger.choice(u'<b>{}</b>] {}'.format(i+1, m.name))



import six
import yaml
import uuid
import re
try:
        # for python newer than 2.7
    from collections import OrderedDict
except ImportError:
        # use backport from pypi
    from ordereddict import OrderedDict
try:
    from yaml import CLoader as YamlLoader, CDumper as YamlDumper
except ImportError:
    from yaml import Loader as YamlLoader, Dumper as YamlDumper

_mapping_tag = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG
dict_representer = lambda self, data: self.represent_mapping('tag:yaml.org,2002:map', data.items())
dict_constructor = lambda loader, node: OrderedDict(loader.construct_pairs(node))
YamlDumper.add_representer(OrderedDict, dict_representer) 
YamlLoader.add_constructor(_mapping_tag, dict_constructor)


__all__ = [
    'setdefaultdict', 
    'memoized_property', 
    'memoized', 
    'yamp_load', 
    'yamp_dump', 
    'yaml', 
    'OrderedDict'
]


def yamp_dump(data):
    return yaml.dump(data, Dumper=YamlDumper, default_flow_style=False, indent=4)

def yamp_load(content):
    return yaml.load(content, Loader=YamlLoader)


def setdefaultdict(_dict, name=None, default={}, ordered=False):
    if name:
        value = _dict.setdefault(name, default)
    else:
        value = _dict
    if not isinstance(value, dict): 
        if name:
            value = _dict[name] = default
        else:
            value = default
    if ordered:
        value = OrderedDict(value)
    return value


        
class memoized_property(object):


    def __init__(self, fget=None, fset=None, fdel=None, doc=None):
        self.__get = fget
        self.__set = fset
        self.__del = fdel
        self.__doc__ = doc
        self.___uuid = str(uuid.uuid1())
    
    def __get__(self, inst, cls=None): 
        if not hasattr(inst, self.___uuid):
            setattr(inst, self.___uuid, self.__get(inst))
        return getattr(inst, self.___uuid)


class memoized(object):

    def __init__(self, fct):
        self.___fct = fct
        self.___uuid = str(uuid.uuid1())

    def __get__(self, inst, cls=None):
        self.___inst = inst
        return self

    def __call__(self, inst, *args, **kwargs):        
        if not hasattr(inst, self.___uuid):
            setattr(inst, self.___uuid, self.___fct(inst, *args, **kwargs))
        return getattr(inst, self.___uuid)


def combine(elm1, elm2, *args):

    # elm1 is merge into elm2 with elm1 as the priority
    # elm2 < elm1

    # None - None
    if elm1 is None and elm2 is None:
        return None

    # None - str
    elif elm1 is None and isinstance(elm2, six.string_types):
        elm1 = elm2

    # None - dict
    elif elm1 is None and isinstance(elm2, dict):
        elm1 = elm2
        combine(elm1, elm2)

    # None - List
    elif elm1 is None and isinstance(elm2, list):
        elm1 = elm2
        combine(elm1, elm2)

    # Dict - Dict
    elif isinstance(elm1, dict) and isinstance(elm2, dict):
        for key2, val2 in elm2.items():
            val1 = elm1.get(key2, val2)
            elm1[key2] = combine(val1, val2)
            # if val1 is not None:
            #     elm1[key2] = val1


    # List - List
    elif isinstance(elm1, list) and isinstance(elm2, list):
        elm1 = list(set(elm2 + elm1))

    # List - None
    elif isinstance(elm1, list) and elm2 is None:
        for ind1, val1 in enumerate(elm1):
            elm1[ind1] = combine(ind1, None)

    # if isinstance(elm1, six.string_types) and elm1 in as_shortcuts:
    #     elm1 = as_shortcuts[elm1]

    if args:
        return combine(elm1, **args)
    else:
        return elm1




import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    
    name = args[0].strip() if args else None
    if name:
        args = list(args)
        args.pop(0)
        mounting = root.project['mounting'].get(name)
        if mounting:
            logger.success(u'Prepare mounting <b>%s</b> to be created.' % mounting.name)
        else:
            logger.error(u'Mounting <b>%s</b> unknow.' % name)
            exit(1)
    else:
        mounting = root.mounting
        
    if not mounting.is_localhost:

        root.run_command('remove', *args, internal=True)
        logger.cmd('Create machine <b>%s</b>' % (mounting.docker_machine_name, ))
        logger.cmd('With driver <b>%s</b>' % (mounting.get_machine_driver(), ))
        cmd = root.bash(
            mounting.docker_machine_bin, 
            'create', 
            '--driver',
            mounting.get_machine_driver(),        
            mounting.docker_machine_name,#root.compose.name,
            *args,
            is_system=True
        )
    else:
        logger.warning(mounting.LOCAL_MACHINE_WARNING)


from docker_emperor.utils import yaml


__all__ = ['Environment']


class Environment(list):

    def __new__(cls, *args, **kwargs):
        return list.__new__(cls, *args, **kwargs)

    def __init__(self, value, *args, **kwargs):
        if not isinstance(value, list):
            if isinstance(value, dict):
                value = ['{}={}'.format(key,val) for key,val in value.items()]
            else:
                value = []
        self.dict = dict()
        for var in value:
            if isinstance(var, tuple):
                key, val = var
            else:
                key, val = var.split('=', 1) 
            self.dict[key.strip()] = val.strip()
        super(self.__class__, self).__init__(value)
    
    def __repr__(self):
        return '<%s: %s\r\n>' % (
            self.__class__.__name__, 
            "".join(["\r\n\t%s: %s" % (key, val) for key,val in self])
        )

    def __gt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        return inst < self

    def __lt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        del self[:]     
        self.dict.update(inst.dict) 
        self.__init__(self.dict)
        return self

    def items(self):
        return self.dict.items()

    @property
    def list(self):
        # e = []
        return [
            '%s=%s' % (k, v.strip().replace('\\ ', '').replace(' ', '\\ ')) 
            for k, v in self
        ]
        #     v = 
        #     # if ' ' in v:
        #     #     s = v.strip('"').replace('"', '\\"')
        #     #     s = '"%s"' % s
        #     # s = 
        #     e.append)
        # return e#[escape_arg('%s=%s' % (k, v)) for k, v in self]

    def __dict__(self, key):
        return self.dict

    def get(self, key, default):
        return self[key] or default

    def __getitem__(self, key):
        return self.dict.get(key)

    def __setitem__(self, key, value):
        self.dict[key] = value
        self.__init__(self.dict)

    def __iter__(self):
        for key, val in self.items():
            yield key, val

    def __set__(self, value):
        del self[:]
        self[:] = value        

    def copy(self):
        return self.__class__(self.dict)
    

yaml.add_representer(Environment, lambda dumper, data: dumper.represent_list(["{}={}".format(key, val) for key, val in data.items()]))


from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    
    Command(
        root.machine.bin, 
        'regenerate-certs', 
        root.machine.name,
        is_system=True
    )


import os
import six
import docker_emperor.logger as logger


__all__ = ['run']


def run(root, *args, **kwargs):


    def select_machine_name():
        # logger.ask('0) no machine (localhost)')
        logger.ask(u'Please select the <b>{}</b> machine to work on'.format(root.project.name))
        for i, m in enumerate(root.project['machines']):
            logger.choice(u'<b>{}</b>] {}'.format(i+1, m.name))
        mi = six.moves.input(': ')
        try:
            if mi == '0':
                raise Exception
            return root.project['machines'][int(mi)-1].name
        except Exception as e:
            logger.error(u'<b>%s</b> is not a valid choice' % mi)
            return select_machine_name()

    root.project.config['machine'] = select_machine_name()
    logger.success(u'Machine <b>%s</b> selected.' % root.machine.name)



from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    
    logger.cmd('Command aliases for <b>%s</b> : \r' % root.project.compose.name)
    for aliase in root.project['aliases']:
        logger.choice(' <b>%s</b>%s%s' % (aliase.name, ' '*(20-len(aliase.name)), aliase))




import six
from docker_emperor.nodes.environment import Environment
from docker_emperor.nodes.service import Services
from docker_emperor.nodes.command import Commands
from docker_emperor.utils import setdefaultdict, combine, memoized_property, memoized, OrderedDict


__all__ = ['Contexts', 'Context']



class Contexts(dict):

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __init__(self, data):
        super(Contexts, self).__init__(OrderedDict(setdefaultdict(data)))
        for key, val in self.items():
            self[key] = Context(key, val)
        # else:
        #     self['default'] = Context('default')

    def __iter__(self):
        for key, val in self.items():
            yield val

    def __repr__(self): 
        return ", ".join(str(c) for c in self)

    def __getitem__(self, i):
        if isinstance(i, int):
            return [c for c in self][i]
        else:
            return self.get(i, None)


class Context(dict):

    COMMANDS = [
    ]

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __init__(self, name, data=dict()):
        self.name = name
        super(Context, self).__init__(setdefaultdict(data))
        for default_name, default_class in [
            ('environment', Environment),
            ('services', Services),
            ('commands', Commands),
        ]:
            self[default_name] = default_class(self[default_name])
        
    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, self.name)

    def __getitem__(self, key): 
        return self.get(key)



import os
import docker_emperor.logger as logger

'''
or docker-machine ssh virtualbox -- tce-load -wi rsync
'''
def run(root, *args, **kwargs):
  
    logger.cmd('Sync files for project <b>%s</b>' % (root.compose.name, ))
    root.run_command('machine:start', internal=True)
    if not root.mounting.is_localhost:

        # ex. docker-machine scp -r -d . virtualbox:/home/docker/project.dev.localhost/
        for file in root.mounting['files']:
            
            cmd = root.bash(
                root.mounting.docker_machine_bin,
                'scp',
                '-r',
                '-d',
                file, 
                '{}:{}'.format(
                    root.mounting.docker_machine_name,#root.compose.name, 
                    root.mounting['workdir']
                ),
                is_system=True,
            )
            print(cmd.cmd_line)
    else:
        logger.warning(root.mounting.LOCAL_MACHINE_WARNING)



import os
from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    root.run_command('machine:start', internal=True)

    mounting = root.mounting
    hosts = mounting.get_machine_hosts()
    if hosts:
        
        for host in hosts:

            ip_address = '0.0.0.0'
            logger.cmd('Set host <b>%s</b> mapped at %s in /etc/hosts.' % (host, ip_address))
            cmd = root.bash(
                "docker", 
                "run", 
                "-t", 
                "-i" ,
                "-v", "%s/managehosts:/bin/managehosts" % root.bin_root,
                "-v", "/etc/hosts:/etc/host_hosts",
                "-v", "/etc/hosts.bak:/etc/host_hosts.bak",
                "busybox",
                "sh", 
                "/bin/managehosts" ,
                "add %s %s %s" % (host, ip_address, root.project.name),
                mounting=mounting,
                is_system=True
            )

            if cmd.is_success:
                logger.success('Host <b>%s</b> has been set on %s.' % (host, mounting, ))



import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    logger.cmd('Build <b>%s</b>' % (root.project.compose.name, ))
    root.run_command('machine:start', internal=True)

    cmd = root.bash(
        root.project.compose.bin, 
        'build',
        *args, 
        compose=root.compose,
        is_system=True
    )
    if cmd.is_success:
        logger.success('<b>%s</b> built.' % (root.project.compose.name, ))



import os
import copy
from docker_emperor.utils import setdefaultdict, yaml


__all__ = ['Volumes', 'Volume']



class Volumes(dict):

    def __new__(cls, data, *args, **kwargs):
        return dict.__new__(cls, data, *args, **kwargs)

    def __init__(self, data, *args):
        super(Volumes, self).__init__(setdefaultdict(data, ordered=True))
        for key ,val in self.items():
            self[key] = Volume(key, val)
        for arg in args:
            self < arg

    def __gt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        return inst < self

    def __lt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        for key, val in inst.items():
            if not key in self:
                self[key] = val.copy()
            else:
                self[key] < val
        return self

    def __iter__(self):
        for name, service in self.items():
            yield service

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, super(dict, self).__repr__())


yaml.add_representer(Volumes, lambda dumper, data: dumper.represent_dict(data))


class Volume(dict):


    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __init__(self, name, data, *args):
        self.name = name
        super(self.__class__, self).__init__(setdefaultdict(data))
        for arg in args:
            self < arg

    def __getitem__(self, key): 
        return self.get(key)

    def __gt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        return inst < self

    def __lt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        for key, val in inst.items():
            if not key in self:
                self[key] = copy.deepcopy(val)
            else:
                if isinstance(self[key], dict) and isinstance(val, dict):
                    self[key].update(val)
                elif isinstance(self[key], list) and isinstance(val, list):
                    self[key] = list(set(self[key] + copy.deepcopy(val)))

        return self

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, self.name)
       
    def copy(self):
        return self.__class__(self.name, dict(self))
    


yaml.add_representer(Volume, lambda dumper, data: dumper.represent_dict(data))


from docker_emperor.utils import setdefaultdict


__all__ = ['Commands', 'CustomCommand']


class Commands(dict):

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __init__(self, data):
        super(self.__class__, self).__init__(setdefaultdict(data))
        for key, val in self.items(): 
            if isinstance(val, list):
                self[key] = CustomCommand_list(key, val)
            else:
                self[key] = CustomCommand_str(key, val)


    def __gt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        return inst < self

    def __lt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        for name, inst in inst.items(): 
            if not name in self:
                self[name] = inst.copy()
            else:
                self[name] < inst
        return self

    def __iter__(self):
        for name, inst in self.items(): 
            yield inst

    def __repr__(self):
        return '<{}> \r\n\t - {}'.format(self.__class__.__name__, "\r\n\t - ".join([repr(a) for a in self ]))

    def copy(self):
        return self.__class__(dict(self))


class CustomCommand():

    def __gt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        return inst < self

    def __lt__(self, inst):
        if not isinstance(inst, self.__class__): return self
        # self.__init__(self.name, combine(inst, self))
        return self

    def __repr__(self):
        return '{}: {}'.format(self.name, self.data)

    def copy(self):
        return self#.__class__(self.name, self.data)


class CustomCommand_str(CustomCommand, str):

    def __new__(cls, value, *args, **kwargs):
        return str.__new__(cls, value)

    def __init__(self, name, data, *args, **kwargs):
        self.name = name
        self.data = data
        super(CustomCommand_str, self).__init__(data)

    @property
    def commands(self):
        return [self.data]


class CustomCommand_list(CustomCommand, list):

    def __new__(cls, value, *args, **kwargs):
        return list.__new__(cls, value, *args)

    def __init__(self, name, data, *args, **kwargs):
        self.name = name
        self.data = data
        super(CustomCommand_list, self).__init__(data)

    @property
    def commands(self):
        return self.data


import os
from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    root.run_command('machine:start', internal=True)
    root.bash(
        root.compose.bin,
        'exec', 
        *args,
        compose=root.compose,
        is_system=True
    )


import six
from docker_emperor.commands import Command
from docker_emperor.nodes.environment import Environment
from docker_emperor.nodes.service import Services
from docker_emperor.nodes.command import Commands
from docker_emperor.nodes.volume import Volumes
from docker_emperor.utils import setdefaultdict, combine, memoized_property, memoized, OrderedDict


__all__ = ['Mountings', 'Mounting']



class Mountings(dict):

    DEFAULT = {
        'localhost': {}
    }

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __init__(self, project, data):
        self.project = project
        super(Mountings, self).__init__(OrderedDict(setdefaultdict(data)))
        if self:
            for key, val in self.items():
                self[key] = Mounting(self.project, key, val)
        else:
            self['localhost'] = Mounting(self.project, 'localhost')

    def __iter__(self):
        for key, val in self.items():
            yield val

    def __repr__(self): 
        return ", ".join(str(c) for c in self)

    def __getitem__(self, i):
        if isinstance(i, int):
            return [c for c in self][i]
        else:
            return self.get(i, None)


class Mounting(dict):

    COMMANDS = [
    ]
    LOCAL_MACHINE_WARNING = 'You are already on a local machine'

    class Drivers(object):
        LOCALHOST = 'localhost'
        GENERIC_LOCALHOST = 'generic --generic-ip-address localhost'  

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __init__(self, project, name, data=dict(), docker_machine_bin="docker-machine"):
        self.project = project
        self.name = name
        self.docker_machine_name = '%s.%s' % (self.project.name, self.name)
        self.docker_machine_bin = docker_machine_bin
        super(Mounting, self).__init__(setdefaultdict(data))
        for default_name, default_class in [
            ('environment', Environment),
            ('services', Services),
            ('commands', Commands),
            ('volumes', Volumes),
        ]:
            self[default_name] = default_class(self[default_name])
        
        if not isinstance(self['driver'], six.string_types): 
            self['driver'] = Mounting.Drivers.LOCALHOST
        if not isinstance(self['hosts'], list): 
            self['hosts'] = []
        if not isinstance(self['files'], list): 
            self['files'] = []
        if not isinstance(self['workdir'], six.string_types): 
            self['workdir'] = '/home/docker/'

        
            
    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, self.name)

    def __getitem__(self, key): 
        return self.get(key)

    def bash(self, *args, **kwargs):        
        cmd = Command(self.docker_machine_bin, *args, **kwargs)
        cmd.run()
        return cmd

    def get_machine_hosts(self): 
        hosts = [] 
        for host in self['hosts']: 
            hosts.append(self.apply_env(host))    
        return hosts

    def get_machine_driver(self):      
        return self.apply_env(self['driver'])

    def apply_env(self, s):
        for k, v in self['environment']:
            s = s.replace('${%s}' % (k, ), v)
        return s

    @property
    def is_localhost(self):
        return self['driver'] == Mounting.Drivers.LOCALHOST

    @property
    def is_generic(self):
        return self['driver'].startswith('generic')

    @property
    def exists(self):
        cmd = self._run("ls", "--filter", "NAME=" + self.docker_machine_name, "--format", "{{.Name}}", machine=self, tty=False)
        for line in cmd.lines:
            if line == self.docker_machine_name:return True
        return False

    @property
    def docker_env(self):
        n = '__docker_env'
        if not hasattr(self, n):
            if self.is_localhost:
                env = []
            else:
                cmd = self.bash('env', self.docker_machine_name)
                starts = 'export '
                env = [line.lstrip(starts) for line in cmd.lines if line.startswith(starts)]
            setattr(self, n, env)
        return getattr(self, n)

    def start(self):
        if self.is_localhost:
            if not self.is_running:
                self.bash('start', self.docker_machine_name, sys=True).run().log()  
        return self.is_running

    @property
    def is_running(self):
        return self.status == 'Running'

    @property
    def is_startable(self):
        return not self.is_localhost and not self.is_generic
    
    
    @property
    def status(self):
        if self.is_localhost:
            return 'Running'
        else:
            return self.bash('status', self.docker_machine_name).out
    
    @property
    def ip(self):
        if self.is_localhost:
            return '0.0.0.0'
        else:
            return self.bash('ip', self.docker_machine_name, log=False, machine=self).out.strip()

    @property
    def pwd(self):
        return self.bash('ssh', self.docker_machine_name, 'pwd', log=False, machine=self).out.strip()

    @property
    def inspect(self):
        return self.bash('inspect', self.docker_machine_name, machine=self, tty=False).out

    @property
    def active(self):
        return self.bash('active', machine=self, tty=False).out

    def remove(self):
        return self.bash('rm', self.docker_machine_name, machine=self)

# active
# config
# create
# env
# help
# inspect
# ip
# kill
# ls
# mount
# provision
# regenerate-certs
# restart
# rm
# scp
# ssh
# start
# stop
# upgrade
# url

Command.Mounting = Mounting


import os
from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    cmd = root.bash(
        root.project.compose.bin,
        'logs',
        *args,
        compose=root.compose,
        is_system=True
    )


import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    
    if not root.mounting.is_running:
        if root.mounting.is_startable:
            logger.cmd('Start mounting <b>%s</b>' % (root.mounting.name, ))
            cmd = root.bash(
                root.mounting.docker_machine_bin, 
                'start', 
                root.mounting.name, 
                *args,
                sys=True
            )
            if root.mounting.is_running:
                root.logger.success('Machine <b>%s</b> started.' % (root.mounting.name, ))
        else:
            root.logger.warning('Machine <b>%s</b> is not startable.' % (root.mounting.name, ))

    return root.mounting.is_running

# class Command():

#     def run(self, *args, **kwargs):
#         if not self.interna
#         self.success('Machine <b>%s</b> started.' % (root.mounting.name, ))



HEADER = '\033[40;223m'
BLUE = '\033[40;38;5;39m'
GREEN = '\033[40;92m'
YELLOW = '\033[40;93m'
ERROR = '\033[40;38;5;9m'
UNDERLINE = '\033[4m'
ORANGE = '\033[40;38;5;214m'

BOLD = '\033[1m'
END = '\033[0m'

class Logger():

    def __init__(self, root):
        self.root = root

    def cmd(self, *value):
        self.log(*value, color=HEADER)

    def success(self, *value):
        self.log(*value, color=GREEN)

    def choice(self, *value):
        self.log(*value, color=BLUE)

    def info(self, *value):
        self.log(*value, color=HEADER)

    def ask(self, *value):
        self.log(*value, color=YELLOW)

    def comment(self, *value):
        self.log(*value, color=HEADER)

    def warning(self, *value):
        self.log(*value, color=ORANGE)

    def error(self, *value):
        self.log(*value, color=ERROR)     

    def log(self, *value, **kwargs):
        color = kwargs.pop('color' ,YELLOW)
        value = " " + " ".join([str(v) for v in value]) + " "
        value = '{}>{}{}'.format(color, value, END)
        value = value.replace('<b>', BOLD).replace('</b>', END + color)
        print(value.strip())

def cmd(*value):
    log(*value, color=HEADER)

def success(*value):
    log(*value, color=GREEN)

def choice(*value):
    log(*value, color=BLUE)

def info(*value):
    log(*value, color=HEADER)

def ask(*value):
    log(*value, color=YELLOW)

def comment(*value):
    log(*value, color=HEADER)

def warning(*value):
    log(*value, color=ORANGE)

def error(*value):
    log(*value, color=ERROR)

def log(*value, **kwargs):
    color = kwargs.pop('color' ,YELLOW)
    value = " " + " ".join([str(v) for v in value]) + " "
    value = '{}>{}{}'.format(color, value, END)
    value = value.replace('<b>', BOLD).replace('</b>', END + color)
    print(value.strip())


import os
import sys
import setuptools

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (2, 7)
EXCLUDE_FROM_PACKAGES = [
    'tests', 'build', 'dist',
]
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================


This version of Django requires Python {}.{}, but you're trying to
install it on Python {}.{}.

""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

version = __import__('docker_emperor').__version__

setuptools.setup(
    name='docker-emperor',
    version=version,
    # python_requires='>={}.{}'.format(*REQUIRED_PYTHON),
    author = "Damien Autrusseau",
    author_email = "damien.autrusseau@gmail.com",
    description = ("Docker CLI that combine compose and machine for a full stack deployment"),
    license = "Apache Software License",
    keywords = "",
    url = "https://pypi.org/project/docker-emperor",
    #packages=['tests'],
    long_description=read('README.rst'),
    packages=setuptools.find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    include_package_data=True,
    install_requires=[
        'PyYAML==3.12',
        'six==1.11.0',
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    entry_points = {
        'console_scripts': [
            'docker-emperor = docker_emperor.root:entrypoint',    
            'de = docker_emperor.root:entrypoint',                
        ],              
    },
)


import os
import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    logger.cmd('Down project <b>%s</b>' % (root.compose.name, ))
    cmd = root.bash(
        root.compose.bin,
        'down',
        '--remove-orphans',
        compose=root.compose,
        is_system=True
    )
    if cmd.is_success:
        logger.success('<b>%s</b> is down.' % (root.compose.name, ))


import six
import os
from docker_emperor.commands import Command
from docker_emperor.nodes.environment import Environment
from docker_emperor.nodes.mounting import Mountings
from docker_emperor.nodes.service import Services
from docker_emperor.nodes.volume import Volumes
from docker_emperor.utils import yamp_dump, yaml


__all__ = ['Compose']

''' 
Compose stack Context + Machine 
Combine services & environment vars
'''

class Compose(dict):


    NODES = [
        'services', 
        'volumes', 
        'networks', 
        'version', 
        'configs', 
        'secrets', 
        'deploy'
    ]

    NOT_NODES = [
        'name', 
        'environment', 
        'commands', 
        'mounting'
    ]

    def __new__(cls, *args, **kwargs):
        return dict.__new__(cls, *args, **kwargs)

    def __repr__(self):
        return '<%s: %s>'.format(self.__class__.__name__, self.name) 

    def __getitem__(self, key): 
        return self.get(key)

    def __init__(self, root, docker_compose_bin="docker-compose"):

        self.root = root
        self.project = self.root.project  
        self.mounting = self.project.mounting
        self.name = self.mounting.docker_machine_name
        self.filename = os.path.join(self.root.root_path, 'docker-compose.%s.yml' % self.name)
        data = dict(self.project)
        for node_name, node in data.items():
            if node_name in Compose.NOT_NODES:
                data.pop(node_name)
        super(Compose, self).__init__(data)
        for default_name, default_class in [
            ('services', Services),
            ('volumes', Volumes),
        ]:
            self[default_name] = default_class(self[default_name])

        self.environment = self.project['environment'].copy()

        self['services'] < self.mounting['services'] 
        self['volumes'] < self.mounting['volumes'] 
        self.environment < self.mounting['environment'] 

        self.environment['DOCKER_EMPEROR_HOSTS'] = " ".join([
            host.strip() for host in self.mounting['hosts']    
        ])
        self.environment['DOCKER_EMPEROR_ENVIRONMENT']  = " ".join(self.environment.list)

        self.bin = "%s -f %s" % (docker_compose_bin, self.filename)
        #self.bin = "%s -f %s --project-directory=%s" % (path, self.filename, self.mounting['workdir'])
        # '--project-directory=.',

        for service in self['services']:

            service['environment'] < self.environment
            service['container_name'] = service.get('container_name', '%s.%s' % (self.name, service.name))
            if not 'image' in service and not 'build' in service:
                if os.path.isfile(os.path.join(self.root.root_path, service.name, 'Dockerfile')):
                    service['image'] =  '%s.%s' % (self.project.name, service.name)
                    service['build'] = service.name
                else:
                    service['image'] = service.name
            else:
                if 'image' in service:
                    if os.path.isfile(os.path.join(self.root.root_path, service['image'], 'Dockerfile')):
                        service_name = service['image']
                        service['image'] = '%s.%s' % (self.project.name, service_name)
                        service['build'] = service_name

            if not self.mounting.is_localhost:
                if 'volumes' in service:

                    new_volumes = []
                    for volume in service['volumes']:
                        # print(volume)

                        volume = volume.strip()
                        if volume.startswith('./'):
                            volume = self.mounting['workdir'] + volume[2:]
                            new_volumes.append(volume)
                        else:
                            new_volumes.append(volume)
                    service['volumes'] = new_volumes





        file = open(self.filename, 'wb')# = tempfile.NamedTemporaryFile(mode='w+b', bufsize=-1, suffix='.yml', prefix='docker-compose-', dir=None, delete=False)
        yml = yamp_dump(self)
        for name, value in self.environment:
            yml = yml.replace('${%s}' % (name), value)
        file.write(yml)
        file.close()

    def copy(self):
        return self.__class__(dict(self))
 
yaml.add_representer(Compose, lambda dumper, data: dumper.represent_dict(data))


Command.Compose = Compose


import docker_emperor.logger as logger

'''
or docker-machine ssh virtualbox -- tce-load -wi rsync
'''
def run(root, *args, **kwargs):

    logger.cmd('Removing all docker containers..')
    root.bash(
        'docker rm $(docker ps -a -q) -f',
        compose=root.compose,
        is_system=True
    )


from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    mounting = root.mounting    
    if mounting.is_localhost:
        logger.warning(mounting.LOCAL_MACHINE_WARNING)
    else:
        cmd = root.bash(
            mounting.docker_machine_bin, 
            'ssh', 
            mounting.docker_machine_name,
            *args,
            is_system=True
        )
        if cmd.is_success:
            logger.success('')



import docker_emperor.logger as logger


def run(root, *args, **kwargs):

    logger.cmd('Rebuild <b>%s</b>' % (root.project.compose.name, ))
    root.project.machine.start()
    cmd = root.bash(
        root.project.compose.bin, 
        '--no-cache',
        'build',
        *args, 
        compose=root.compose,
        is_system=True
    )
    if cmd.is_success:
        logger.success('<b>%s</b> rebuilt.' % (root.project.compose.name, ))



from docker_emperor.commands import Command
import docker_emperor.logger as logger


def run(root, *args, **kwargs):
    
    status = Command(
        root.machine.bin, 
        'status', 
        root.machine.name
    ).out

    logger.warning(status)


import docker_emperor.logger as logger

'''
or docker-machine ssh virtualbox -- tce-load -wi rsync
'''
def run(root, *args, **kwargs):

    root.bash(
        *args,
        compose=root.compose,
        is_system=True
    )


import os
from docker_emperor.commands import Command
import docker_emperor.logger as logger

'''
or docker-machine ssh virtualbox -- tce-load -wi rsync
'''
def run(root, *args, **kwargs):
    
    for sys in ['system', 'network', 'volume']:
        root.bash(
            root.docker_path,
            sys,
            'prune',
            compose=root.compose,
            is_system=True
        )